@@ -83,7 +83,11 @@ static psm2_error_t spio_credit_return_update_shared(struct ips_spio *ctrl);
 
 static PSMI_HAL_INLINE psm2_error_t
 ips_spio_init(const struct psmi_context *context, struct ptl *ptl,
-	      struct ips_spio *ctrl)
+	      struct ips_spio *ctrl
+#ifdef PSM_AVX512
+	      , int is_avx512_enabled
+#endif
+	      )
 {
 	cpuid_t id;
 	hfp_gen1_pc_private *psm_hw_ctxt = context->psm_hw_ctxt;
@@ -170,8 +174,8 @@ ips_spio_init(const struct psmi_context *context, struct ptl *ptl,
 
 #ifdef PSM_AVX512
 	/* 64B copying supported */
-	ctrl->spio_blockcpy_large = (id.ebx & (1<<AVX512F_BIT)) ?
-		hfi_pio_blockcpy_512 : ctrl->spio_blockcpy_med;
+	ctrl->spio_blockcpy_large = (is_avx512_enabled && (id.ebx & (1<<AVX512F_BIT))) ?
+		hfi_pio_blockcpy_512 : ctrl->spio_blockcpy_large;
 
 #endif
 
@@ -636,7 +640,7 @@ ips_spio_process_events(const struct ptl *ptl_gen)
 
 	if (event_mask & PSM_HAL_HFI_EVENT_SL2VL_CHANGE) {
 		_HFI_INFO("SL2VL mapping changed for port.\n");
-		ips_ibta_init_sl2sc2vl_table(&((struct ptl_ips *)(ctrl->ptl))->proto);
+		ips_ibta_init_sl2sc_table(&((struct ptl_ips *)(ctrl->ptl))->proto);
 	}
 
 	return PSM2_OK;
@@ -682,6 +686,7 @@ ips_spio_transfer_frame(struct ips_proto *proto, struct ips_flow *flow,
 	if (do_lock)
 		pthread_spin_lock(&ctrl->spio_lock);
 
+#ifdef PSM_FI
 	if_pf(PSMI_FAULTINJ_ENABLED()) {
 		PSMI_FAULTINJ_STATIC_DECL(fi_lost, "piosend", 1,
 					  IPS_FAULTINJ_PIOLOST);
@@ -695,6 +700,7 @@ ips_spio_transfer_frame(struct ips_proto *proto, struct ips_flow *flow,
 			goto fi_busy;
 		/* else fall through normal processing path, i.e. no faults */
 	}
+#endif /* #ifdef PSM_FI */
 
 	psmi_assert((length & 0x3) == 0);
 	paylen = length + (cksum_valid ? PSM_CRC_SIZE_IN_BYTES : 0);
@@ -705,7 +711,9 @@ ips_spio_transfer_frame(struct ips_proto *proto, struct ips_flow *flow,
 
 		if_pf(spio_ctrl->spio_available_blocks < nblks) {
 			/* Check unit status */
+#ifdef PSM_FI
 fi_busy:
+#endif /* #ifdef PSM_FI */
 			if ((err =
 			     psmi_context_check_status(ctrl->context)) ==
 			    PSM2_OK) {
